<?php

use Imoje\Payment\Api;
use Imoje\Payment\Util;

/**
 * Class WC_Gateway_ImojeBlik
 */
class WC_Gateway_ImojeBlik extends WC_Payment_Gateway
{

	/**
	 * @var string
	 */
	const PAYMENT_METHOD = 'imoje_blik';

	/**
	 * initialise gateway with custom settings
	 */
	public function __construct()
	{

		$this->setup_properties();

		$this->title = $this->get_option('title') ?: 'BLIK';

		$this->init_form_fields();
		$this->init_settings();

		$this->icon = $this->get_option('hide_brand') === 'no'
			? WOOCOMMERCE_IMOJE_PLUGIN_URL . 'resources/images/blik.svg'
			: null;

		// region actions
		add_action('woocommerce_update_options_payment_gateways_' . $this->id, [
			$this,
			'process_admin_options',
		]);
		add_action('woocommerce_receipt_' . $this->id, [
			$this,
			'receipt_page',
		]);
		add_action('woocommerce_api_' . strtolower(get_class($this)), [
			$this,
			'imoje_blik_api',
		]);
		// endregion

		// region filters
		add_filter('woocommerce_available_payment_gateways', [
			$this,
			'check_imoje_blik_available_payment_gateways',
		]);
		// endregion
	}

	/**
	 * Setup general properties for the gateway.
	 */
	protected function setup_properties()
	{

		$this->id = self::PAYMENT_METHOD;
		$this->method_title = __('imoje - BLIK', 'imoje');
		$this->method_description = __('BLIK payments', 'imoje');
		$this->has_fields = false;
		$this->supports = [
			'products',
			'refunds',
		];
	}

	/**
	 * Initialize form in admin panel.
	 */
	public function init_form_fields()
	{

		$currencies = Util::getSupportedCurrencies();

		$this->form_fields = [
			'hint'                => [
				'title'       => __('Hint', 'imoje'),
				'class'       => 'hidden',
				'type'        => 'title',
				'description' => __('The module requires a configuration in the imoje administration panel. <br/> Go to imoje.ing.pl and log in to the administration panel. <br/> Go to stores tab and enter the notification address in the appropriate field and copy the configuration keys. <br/> Copy the keys into the fields described below.', 'imoje'),
			],
			'enabled'             => [
				'title'   => __('Enable / Disable', 'imoje'),
				'type'    => 'checkbox',
				'label'   => __('Enable', 'imoje'),
				'default' => 'no',
			],
			'debug_mode'          => [
				'title'   => __('Debug mode', 'imoje'),
				'type'    => 'checkbox',
				'label'   => __('Enable', 'imoje'),
				'default' => 'no',
			],
			'sandbox'             => [
				'title'   => __('Sandbox', 'imoje'),
				'type'    => 'checkbox',
				'default' => 'no',
				'label'   => __('Enable', 'imoje'),
			],
			'hide_brand'          => [
				'title'   => __('Display brand', 'imoje'),
				'type'    => 'checkbox',
				'default' => 'no',
				'label'   => __('Hide brand', 'imoje'),
			],
			'view_field'          => [
				'title'   => __('Display field', 'imoje'),
				'type'    => 'checkbox',
				'default' => 'no',
				'label'   => __('Display field on the checkout', 'imoje'),
			],
			//			'oneclick'            => [
			//				'title'   => __('Activate OneClick', 'imoje'),
			//				'type'    => 'checkbox',
			//				'default' => 'no',
			//				'label'   => __('Enable', 'imoje'),
			//			],
			'ing_ksiegowosc'      => [
				'title'   => __('ING Księgowość', 'imoje'),
				'type'    => 'checkbox',
				'default' => 'no',
				'label'   => __('Enable', 'imoje'),
			],
			'title'               => [
				'title'   => __('Payment title', 'imoje'),
				'type'    => 'text',
				'default' => 'BLIK',
			],
			'merchant_id'         => [
				'title'   => __('Merchant ID', 'imoje'),
				'type'    => 'text',
				'default' => '',
			],
			'service_id'          => [
				'title'   => __('Service ID', 'imoje'),
				'type'    => 'text',
				'default' => '',
			],
			'service_key'         => [
				'title'   => __('Service Key', 'imoje'),
				'type'    => 'text',
				'default' => '',
			],
			'authorization_token' => [
				'title'   => __('Authorization token', 'imoje'),
				'type'    => 'text',
				'default' => '',
			],
			'currencies'          => [
				'title'   => __('Currency', 'imoje'),
				'type'    => 'multiselect',
				'class'   => 'wc-enhanced-select',
				'options' => $currencies,
			],
			[
				'title'       => __('Your addres for notifications', 'imoje') . ': ',
				'type'        => 'title',
				'description' => $this->get_api_url(),
			],
		];
	}

	/**
	 * @return string
	 */
	private function get_api_url()
	{
		return add_query_arg('wc-api', 'wc_gateway_imojeblik', home_url('/'));
	}

	/**
	 * @param int    $order_id
	 * @param null   $amount
	 * @param string $reason
	 *
	 * @return bool
	 * @throws Exception
	 */
	public function process_refund($order_id, $amount = null, $reason = '')
	{

		if($amount <= 0) {

			throw new Exception(__('Refund amount must be higher than 0', 'imoje'));
		}

		include_once 'Helper.php';

		return Helper::process_refund(
			$order_id,
			$this->get_option('authorization_token'),
			$this->get_option('merchant_id'),
			$this->get_option('service_id'),
			$amount
		);
	}

	/**
	 * @param int $order_id
	 *
	 * @return array
	 */
	public function process_payment($order_id)
	{

		$order = new WC_Order($order_id);

		// Return thank you redirect
		return [
			'result'   => 'success',
			'redirect' => $order->get_checkout_payment_url(true),
		];
	}

	/**
	 * @param string $order_id
	 *
	 * @throws Exception
	 */
	public function receipt_page($order_id)
	{

		header("Cache-Control: no-cache, no-store, must-revalidate");
		global $wp_query;

		$order = new WC_Order($order_id);

		$api = new Api(
			$this->get_option('authorization_token'),
			$this->get_option('merchant_id'),
			$this->get_option('service_id'),
			$this->get_option('sandbox') === "yes"
				? Util::ENVIRONMENT_SANDBOX
				: Util::ENVIRONMENT_PRODUCTION
		);

		if($this->get_option('view_field') === "yes") {

			$active_profile = [];
			$profile_id = '';

			$is_blik = false;
			//			$is_blik = $this->get_option('oneclick') === "yes"
			//				&& is_user_logged_in();

			if($is_blik) {

				$user_id = get_current_user_id();
				$user = get_userdata($user_id);

				$profileList = $api->getBlikProfileList(
					Util::getCid($user_id . $user->user_email)
				);

				if((isset($profileList['success']) && $profileList['success'])
					&& $profileList['body']['blikProfiles']) {

					foreach($profileList['body']['blikProfiles'] as $profile) {

						if($profile['isActive']) {

							$isProfileLabel = isset($profile['aliasList'][1]);

							foreach($profile['aliasList'] as $k => $v) {

								if(!$isProfileLabel) {

									$profile['aliasList'][$k]['label'] = $profile['label'];
								}
							}

							$profile_id = $profile['id'];
							$active_profile = $profile;
							break;
						}
					}
				}
			}

			$wp_query->query_vars['profile_id'] = $profile_id;
			$wp_query->query_vars['active_profile'] = $active_profile;
			$wp_query->query_vars['payment_link'] = admin_url('admin-ajax.php');
			$wp_query->query_vars['order_id'] = $order_id;
			$wp_query->query_vars['return_url'] = $order->get_checkout_order_received_url();
			$wp_query->query_vars['method'] = self::PAYMENT_METHOD;
			$wp_query->query_vars['oneclick'] = $is_blik;

			load_template(__DIR__ . '/templates/blik/field.php', false);

			die();
		}

		include_once 'Helper.php';

		$transaction = $api->createTransaction(
			$api->prepareData(
				$order->get_total(),
				$order->get_currency(),
				$order_id,
				Util::getPaymentMethod('blik'),
				Util::getPaymentMethodCode('blik'),
				$this->get_return_url($order),
				$this->get_return_url($order),
				$order->get_billing_first_name(),
				$order->get_billing_last_name(),
				$order->get_billing_email(),
				Api::TRANSACTION_TYPE_SALE,
				'',
				'',
				[],
				'',
				Helper::get_invoice($order, $this->get_option('ing_ksiegowosc') === 'yes', true)
			)
		);

		if(!$transaction['success']) {
			wp_redirect(home_url());
		}

		$order_form = $api->buildOrderForm(
			$transaction,
			__('Continue', 'imoje')
		);

		$wp_query->query_vars['order_form'] = $order_form;

		$wc = WC();

		// Remove cart
		$wc->cart->empty_cart();

		load_template(__DIR__ . '/templates/redirect.php', false);
	}

	/**
	 * Method that checking notify and changing status
	 *
	 * @throws Exception
	 */
	public function imoje_blik_api()
	{

		include_once 'Helper.php';

		Helper::check_notification(
			$this->get_option('service_id'),
			$this->get_option('service_key'),
			$this->get_option('debug_mode')
		);
	}

	/**
	 * Validate data on checkout page. Remove imoje when there is an error.
	 *
	 * @param array $gateways
	 *
	 * @return array
	 */
	public function check_imoje_blik_available_payment_gateways($gateways)
	{

		if(is_admin() || !is_checkout()) {

			return $gateways;
		}

		$currencies = $this->get_option('currencies', []);

		if(!is_array($currencies)) {
			unset($gateways['imoje_blik']);

			return $gateways;
		}

		if(!in_array(strtolower(get_woocommerce_currency()), $currencies)
			|| $this->get_option('enabled') === 'no'
			|| empty($this->get_option('authorization_token'))
			|| empty($this->get_option('merchant_id'))
			|| empty($this->get_option('service_id'))) {

			unset($gateways['imoje_blik']);
		}

		return $gateways;
	}
}
